cap program drop reweight_est
program define reweight_est, rclass

syntax varlist, [reweight] [nocct] [ptestimaterun]
	
	tokenize `varlist'
	local yvar "`1'"
	
	if ("`reweight'"=="reweight") {	
			tempfile ready_to_reweight
			qui save `ready_to_reweight', replace
			
				sort townid year j sid 
				collapse townid (mean) tpj = tpj (sd) sd_tpj = tpj, by(tpjid j year)
				summ sd_tpj
				drop sd_tpj
				sort tpjid
				sort townid tpjid
				mkmat townid tpj tpjid j year, matrix(tpjs)
			
			use `ready_to_reweight', clear

			qui svmat tpjs
			rename (tpjs1 tpjs2 tpjs3 tpjs4 tpjs5) (townid_eval tpj_eval tpjid_eval j_eval year_eval)
			
			gen ones = 1
			
				tempfile reweighting_computations
				
				su townid, meanonly
				local numtowns = r(max)
			
				qui gen m00 =.
				qui gen m10 =.
				qui gen m01 =.
				qui gen m11 =.
				
				qui gen f00 =.
				qui gen f10 =.
				qui gen f01 =.
				qui gen f11 =.
				qui gen F11 =.
				qui gen F01 =.
				qui gen F10 =.
				qui gen F00 =.
								
				forval town=1(1)`numtowns' {
					di "Townid `town' of `numtowns'"
					
					qui gen tpj_eval_copy = tpj_eval
					qui replace tpj_eval = . if townid_eval != `town'
					
					qui gen temp = (townid==`town')
					
					qui count if after==0 & cohort==0
					if r(N) > 1 {
						cap lpoly temp grade if after==0 & cohort==0, at(tpj_eval) generate(x00 m00t) nograph
						if  _rc != 0{
							di as result "Error in local polynomial estimation for town `town' in bootstrap `b', 00"
							count if after==0 & cohort==0
							qui gen m00t=.
						}
						else{
							drop x00
						}
					}
					else{
						qui gen m00t=.
					}
					qui count if after==1 & cohort==0
					if r(N) > 1 {
						cap lpoly temp grade if after==1 & cohort==0, at(tpj_eval) generate(x10 m10t) nograph
						if  _rc != 0{
							di as result "Error in local polynomial estimation for town `town' in bootstrap `b', 10"
							count if after==1 & cohort==0
							qui gen m10t=.
						}
						else{
							drop x10
						}
					}
					else{
						qui gen m10t=.
					}
					qui count if after==0 & cohort==1
					if r(N) > 1 {
						cap lpoly temp grade if after==0 & cohort==1, at(tpj_eval) generate(x01 m01t) nograph
						if  _rc != 0{
							di as result "Error in local polynomial estimation for town `town' in bootstrap `b', 01"
							count if after==0 & cohort==1
							qui gen m01t=.
						}
						else{
							drop x01
						}
					}
					else{
						qui gen m01t=.
					}
					qui count if after==1 & cohort==1
					if r(N) > 1 {
						cap lpoly temp grade if after==1 & cohort==1, at(tpj_eval) generate(x11 m11t) nograph
						if  _rc != 0{
							di as result "Error in local polynomial estimation for town `town' in bootstrap `b', 11"
							count if after==1 & cohort==1
							qui gen m11t=.
						}
						else{
							drop x11
						}
					}
					else{
						qui gen m11t=.
					}
					
					
					
					capture kdensity grade if after==0 & cohort==0, at(tpj_eval) generate(x002 f00t) nograph
						if  _rc != 0{
							di as result "Error in kernel density estimation for town `town' in bootstrap `b',00"
							count if after==0 & cohort==0
							qui gen f00t=.
						}
						else{
							drop x002	
						}		
					qui count if after==0 & cohort==0 & townid==`town'
					if r(N) > 1 {
						capture akdensity grade if after==0 & cohort==0 & townid==`town', at(tpj_eval) cdf(F00t) nograph noadaptive
						if  _rc != 0{
							di as result "Error in kernel CDF estimation for town `town' in bootstrap `b',00"
							count if after==0 & cohort==0 & townid==`town'
							qui gen F00t=.
						}
					}
					else{
						qui gen F00t=.
					}
					
					
					
					capture kdensity grade if after==1 & cohort==0, at(tpj_eval) generate(x102 f10t) nograph
						if  _rc != 0{
							di as result "Error in kernel density estimation for town `town' in bootstrap `b',10"
							count if after==1 & cohort==0
							qui gen f10t=.
						}
						else{
							drop x102	
						}	
					qui count if after==1 & cohort==0 & townid==`town'
					if r(N) > 1 {	
						capture akdensity grade if after==1 & cohort==0 & townid==`town', at(tpj_eval) cdf(F10t) nograph noadaptive
						if  _rc != 0{
							di as result "Error in kernel CDF estimation for town `town' in bootstrap `b',10"
							count if after==1 & cohort==0 & townid==`town'
							qui gen F10t=.
						}
					}
					else{
						qui gen F10t=.
					}
					
					
					
					capture kdensity grade if after==0 & cohort==1, at(tpj_eval) generate(x012 f01t) nograph
						if  _rc != 0{
							di as result "Error in kernel density estimation for town `town' in bootstrap `b',01"
							count if after==0 & cohort==1
							qui gen f01t=.
						}
						else{
							drop x012
						}		
					qui count if after==0 & cohort==1 & townid==`town'
					if r(N) > 1 {
						capture akdensity grade if after==0 & cohort==1 & townid==`town', at(tpj_eval) cdf(F01t) nograph noadaptive
						if  _rc != 0{
							di as result "Error in kernel CDF estimation for town `town' in bootstrap `b',01"
							count if after==0 & cohort==1 & townid==`town'
							qui gen F01t=.
						}
					}
					else{
						qui gen F01t=.
					}
					
					
					
					capture kdensity grade if after==1 & cohort==1, at(tpj_eval) generate(x112 f11t) nograph
						if  _rc != 0{
							di as result "Error in kernel density estimation for town `town' in bootstrap `b',11"
							count if after==1 & cohort==1
							qui gen f11t=.
						}
						else{
							drop x112	
						}
					qui count if after==1 & cohort==1 & townid==`town'
					if r(N) > 1 {
						capture akdensity grade if after==1 & cohort==1 & townid==`town', at(tpj_eval) cdf(F11t) nograph noadaptive
						if  _rc != 0{
							di as result "Error in kernel CDF estimation for town `town' in bootstrap `b',11"
							count if after==1 & cohort==1 & townid==`town'
							qui gen F11t=.
						}
					}
					else{
						qui gen F11t=.
					}
										
					qui replace m00 = m00t if townid_eval == `town'
					qui replace m10 = m10t if townid_eval == `town'
					qui replace m01 = m01t if townid_eval == `town'
					qui replace m11 = m11t if townid_eval == `town'
					
					qui replace f00 = f00t if townid_eval == `town'
					qui replace f10 = f10t if townid_eval == `town'
					qui replace f01 = f01t if townid_eval == `town'
					qui replace f11 = f11t if townid_eval == `town'
					
					qui replace F00 = F00t if townid_eval == `town'
					qui replace F10 = F10t if townid_eval == `town'
					qui replace F01 = F01t if townid_eval == `town'
					qui replace F11 = F11t if townid_eval == `town'
					
					drop temp tpj_eval
					drop m00t m10t m01t m11t f00t f10t f01t f11t F00t F10t F01t F11t			 
					
					rename tpj_eval_copy tpj_eval
				}
				
				qui keep if tpjid_eval!=.
				
				*Get sums of sbc
				gen s00 = m00*f00
				qui summ s00 if year_eval == 1990
				scalar S00 = r(sum)
				
				gen s10 = m10*f10
				qui summ s10 if year_eval == 1990
				scalar S10 = r(sum)
				
				gen s01 = m01*f01
				qui summ s01 if year_eval == 1991
				scalar S01 = r(sum)
				
				gen s11 = m11*f11
				qui summ s11 if year_eval == 1991
				scalar S11= r(sum)
				
				drop s00 s01 s10 s11
				
				keep townid_eval j_eval year_eval tpjid_eval tpj_eval m00 m10 m01 m11 f00 f10 f01 f11 F00 F10 F01 F11
				sort townid_eval tpjid_eval
				egen marketid_eval = group(townid_eval year_eval)
				
				xtset marketid_eval j_eval
				di as result "Imputing counterfactual distributions"
				qui gen deltaF00 = F00 - L.F00	
				qui gen deltaF10 = F10 - L.F10	
				qui gen deltaF01 = F01 - L.F01	
				qui gen deltaF11 = F11 - L.F11	
								
				by marketid_eval: replace deltaF00 = F00 if _n==1
				by marketid_eval: replace deltaF10 = F10 if _n==1
				by marketid_eval: replace deltaF01 = F01 if _n==1
				by marketid_eval: replace deltaF11 = F11 if _n==1				
				
				if ("`ptestimaterun'"=="ptestimaterun") {	
					gen deltaF = deltaF10 + deltaF01 - deltaF11				
						count if deltaF<0
		
					scatter deltaF tpj_eval, yline(0) ytitle("imputed density") xtitle("cutoff") graphregion(color(white)) bgcolor(white)
					graph export "${outputfolder}imputed_density.pdf", as(pdf) replace
						
						gen deltatpj = tpj_eval - L.tpj_eval
						
						qui save "${outputfolder}ptest_reweighting_computations.dta", replace
					drop deltaF
					}
							
					drop marketid_eval
					rename tpjid_eval tpjid
			qui save `reweighting_computations', replace
			

			use `ready_to_reweight', clear
			
				merge m:1 tpjid using `reweighting_computations'
				drop _merge
				count if tpj != tpj_eval
				count if tpj != tpj_eval & tpj_eval!=.

				gen m_bc = .
				qui replace m_bc = m00 if after==0 & cohort==0
				qui replace m_bc = m10 if after==1 & cohort==0
				qui replace m_bc = m01 if after==0 & cohort==1
				qui replace m_bc = m11 if after==1 & cohort==1
				
				gen f_bc = .
				qui replace f_bc = f00/scalar(S00) if after==0 & cohort==0
				qui replace f_bc = f10/scalar(S10) if after==1 & cohort==0
				qui replace f_bc = f01/scalar(S01) if after==0 & cohort==1
				qui replace f_bc = f11/scalar(S11) if after==1 & cohort==1	
				
				gen deltaF = deltaF10 + deltaF01 - deltaF11
				replace deltaF = deltaF00 if after==0 & cohort==0					
				gen weighta = deltaF*m00/(f_bc*m_bc)
					
				summ weighta
				gen yvarrw = `yvar'*weighta

			do "${adofolder}rdob_mod2.ado"
				rdob_mod2 yvarrw dzag
			global bw=r(h_opt)
					
			areg yvarrw dga wanted dga_wanted dzag dzag_dga dzag_wanted dzag_dga_wanted cohort dga_cohort dzag_cohort dzag_dga_cohort  before dga_before dzag_before dzag_dga_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			return scalar effect = _b[dga_wanted]
			return scalar effect_a = _b[dga]
			return scalar effect_b = _b[wanted]
			return scalar samplesize = e(N)
			return scalar numclusters = e(N_clust)
			
			areg yvarrw dga wanted dga_wanted dzag dzag_dga dzag_wanted cohort dga_cohort dzag_cohort before dga_before dzag_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			
			return scalar effect_res = _b[dga_wanted]
			return scalar effect_a_res = _b[dga]
			return scalar effect_b_res = _b[wanted]
			return scalar samplesize_res = e(N)
			return scalar numclusters_res = e(N_clust)
			
			if ("`nocct'"!="nocct") {
				rdbwselect yvarrw dzag, covs(wanted  before dzag_wanted  dzag_before cohort  dzag_cohort) c(0)
				global bw=e(h_mserd)
				
				areg yvarrw dga wanted dga_wanted dzag dzag_dga dzag_wanted dzag_dga_wanted cohort dga_cohort dzag_cohort dzag_dga_cohort  before dga_before dzag_before dzag_dga_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
				return scalar effect_cct = _b[dga_wanted]
				return scalar effect_a_cct= _b[dga]
				return scalar effect_b_cct = _b[wanted]
				return scalar samplesize_cct = e(N)
				return scalar numclusters_cct = e(N_clust)
				
				areg yvarrw dga wanted dga_wanted dzag dzag_dga dzag_wanted cohort dga_cohort dzag_cohort before dga_before dzag_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
				
				return scalar effect_res_cct = _b[dga_wanted]
				return scalar effect_a_res_cct = _b[dga]
				return scalar effect_b_res_cct = _b[wanted]
				return scalar samplesize_res_cct = e(N)
				return scalar numclusters_res_cct = e(N_clust)
				
			}
			
			drop m00 m01 m10 m11 m_bc f11 F11 weighta yvarrw
			drop townid_eval tpj_eval j_eval year_eval
		}
		else{
			
			do "${adofolder}rdob_mod2.ado"
				rdob_mod2 `yvar' dzag
			global bw=r(h_opt)
			
			areg `yvar' dga wanted dga_wanted dzag dzag_dga dzag_wanted dzag_dga_wanted cohort dga_cohort dzag_cohort dzag_dga_cohort  before dga_before dzag_before dzag_dga_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			return scalar effect = _b[dga_wanted]
			return scalar effect_a = _b[dga]
			return scalar effect_b = _b[wanted]
			return scalar samplesize = e(N)
			return scalar numclusters = e(N_clust)
			
			areg `yvar' dga wanted dga_wanted dzag dzag_dga dzag_wanted cohort dga_cohort dzag_cohort before dga_before dzag_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			
			return scalar effect_res = _b[dga_wanted]
			return scalar effect_a_res = _b[dga]
			return scalar effect_b_res = _b[wanted]
			return scalar samplesize_res = e(N)
			return scalar numclusters_res = e(N_clust)
			
			if ("`nocct'"!="nocct") {
				rdbwselect `yvar' dzag , covs(wanted  before dzag_wanted  dzag_before cohort  dzag_cohort) c(0)
				global bw=e(h_mserd)
				
				areg `yvar' dga wanted dga_wanted dzag dzag_dga dzag_wanted dzag_dga_wanted cohort dga_cohort dzag_cohort dzag_dga_cohort  before dga_before dzag_before dzag_dga_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			return scalar effect_cct = _b[dga_wanted]
			return scalar effect_a_cct = _b[dga]
			return scalar effect_b_cct = _b[wanted]
			return scalar samplesize_cct = e(N)
			return scalar numclusters_cct = e(N_clust)
			
			areg `yvar' dga wanted dga_wanted dzag dzag_dga dzag_wanted cohort dga_cohort dzag_cohort before dga_before dzag_before if  dzag>=-$bw  & dzag<$bw & dzag~=0, robust cluster(sid2) absorb(uazY)
			
			return scalar effect_res_cct = _b[dga_wanted]
			return scalar effect_a_res_cct = _b[dga]
			return scalar effect_b_res_cct = _b[wanted]
			return scalar samplesize_res_cct = e(N)
			return scalar numclusters_res_cct = e(N_clust)
			
			}
			
		}
   
end
